package Msn::Authentication;

# $Id: Authentication.pm 121 2004-07-17 01:52:00Z tea $

use strict;
use warnings;
use Carp;

use URI;
use HTTP::Request;
use LWP::UserAgent;

use Msn;
use Msn::Util;
use Msn::Error;
use Msn::Connection;


sub _auth {
    my ($conn, $user) = @_;
    
    $conn->sendCommand("VER", "MSNP8", "CVR0");
    $conn->getLine();
    $conn->sendCommand("CVR", MSN_USER_AGENT, $user);
    $conn->getLine();
    $conn->sendCommand("USR", "TWN", "I", $user);
    
    my  $result = $conn->getLine();
    
    if ($result =~ /^(\d+)\b/) {
        confess Msn::Error::getErrorMessage($1);
    }
    
    return split(/\s+/, $result);
}

sub login($$$) {
    my ($package, $user, $pass) = @_;
    
    my $conn = Msn::Connection->new(MSN_DISPATCH_SERVER);
    my $notification_server = (_auth($conn, $user))[3];
    $conn->close();
    
    $conn = Msn::Connection->new($notification_server);
    my %param = split_param((_auth($conn, $user))[4]);
    my $ticket = passport($user, $pass, \%param);
    
    croak "No passport\n" unless $ticket;  
    
    $conn->sendCommand("USR", "TWN", "S", $ticket);
    $conn->getLine();

    # TODO parse MSG Hotmail and set to the client.
    # move this code to MSN.pm, This is not Msn::Authentication's job.
    my $msg_length = (split/\s+/, $conn->getLine())[-1];
    $conn->getMessage($msg_length);

    $conn;
}


sub passport($$$) {
    my ($user, $pass, $param) = @_;
    my ($user_agent, $response, $request);
    
    $user_agent = LWP::UserAgent->new(timeout => 30, agent => "MSNMSG");

    # 1)
    $request = HTTP::Request->new(GET => PASSPORT_URI);
    $request->push_header(Host => URI->new(PASSPORT_URI)->host);
    $response = $user_agent->request($request);

    my %nexus_param = split_param($response->header("PassportURLs"));
    my %auth = (
        "OrgVerb"   => "GET",
        "OrgURL"    => 'http://messenger.msn.com',
        "sign-in"   => $user,
        "pwd"       => $pass,
        "lc"        => $param->{"lc"},
        "id"        => $param->{"id"},
        "tw"        => $param->{"tw"},
        "fs"        => $param->{"fs"},
        "ru"        => $param->{"ru"},
        "ct"        => $param->{"ct"},
        "kpp"       => $param->{"kpp"},
        "kv"        => $param->{"kv"},
        "ver"       => $param->{"ver"},
        "tpf"       => $param->{"tpf"}
    );

    my $auth = join_param(\%auth);

    # 2)
    $request = HTTP::Request->new("GET", "https://".$nexus_param{DALogin});
    $request->authorization("Passport1.4 $auth");
    $response = $user_agent->request($request);

    if ($response->status_line =~ /^200\s+OK/) {
        return
          ($response->header("authentication-info") =~ /\b(t=[^,]+)/)[0];
    }
    
    return undef;
}

1;
