package Msn::Command;

# $Id: Command.pm 102 2004-06-06 15:30:35Z tea $

use strict;
use warnings;
use Carp;
use URI::Escape;

sub new {
    bless {}, shift
}

sub _command {
    my ($self, $cmd, @args) = @_;
    croak "Not set notification connection\n" unless $self->{notification};
    $self->{notification}->sendCommand($cmd, @args);
}

sub getLine {
    my ($self) = @_;
    croak "Not set notification connection\n" unless $self->{notification};
    return $self->{notification}->getLine();
}

sub ping {
    my ($self) = @_;
    # send PNG without TrID
    $self->{notification}->sendRaw("PNG", "\r\n");
}

sub changeState {
    my ($self, $state) = @_;
    $self->_command("CHG", $state);
}

sub online {
    my ($self) = @_;
    $self->changeState("NLN", "0");
}

sub hidden {
    my ($self) = @_;
    $self->changeState("HDN");
}

sub offline {
    my ($self) = @_;
    $self->hidden();
}

sub rename {
    my ($self, $new_name) = @_;
    $self->_command("REA", $self->getUsername(), uri_escape($new_name));
}

sub sync {
    my ($self) = @_;
    $self->_command("SYN", "0", "0");
}

sub logout {
    my ($self) = @_;
    $self->_command("OUT");
    # connection will be closed by host.
    $self->{notification} = undef;
}


1;
