#!/usr/local/bin/perl -w

# $Id: msn_bot.pl 123 2004-07-17 14:03:42Z tea $

use strict;
use warnings;

use URI::Escape;

use Msn::Bot;
use Msn::Bot::Translate;
use Msn::Bot::Google;
use Msn::Bot::Dict;

die "Usage: username password\n" unless @ARGV == 2;

my $username = shift(@ARGV);
my $password = shift(@ARGV);
my $msn = Msn::Bot->new($username, $password);

# FIXME adhoc veriable
$msn->{dump_mode} = 0;
$msn->{translate_mode} = "on";

$msn->setMessageHandlers({
    Status  => \&status_handler,
    Message => \&message_handler,
    Join    => \&join_handler,
    Answer  => \&answer_handler,
})->run();


sub status_handler {
    my ($self, $username, $status) = @_;
    
    print "status: @_\n";
}

sub message_handler {
  my ($self, $username, $nickname, $message) = @_;

  return unless $message;

  # Quit
  if ($message =~ /^quit$/ and $username =~ /^teamikl\@hotmail\.com$/) {
      $self->sendMessage("Bye");
      $self->sendCommand("BYE");
      sleep(5);
      exit(0);
  }
  
  # Dump mode
  if ($message =~ m/DUMP\((.*)\)/ig) {
    $self->{dump_mode} = $1;
    $self->sendMessage("Dump mode: $self->{dump_mode}");
    return;
  }
  if ($self->{dump_mode} eq 'on') {
    $message = join(' ', map { sprintf("%X", $_) } unpack("C*", $message)); 
    $self->sendMessage(qq/$nickname said "$message"/);
    return;
  }

  # Translate mode
  if ($message =~ m/TRANSLATE\((.*)\)/ig) {
    $self->{translate_mode} = $1;
    $self->sendMessage("Translate mode: $self->{translate_mode}");
    return;
  }
  if ($self->{translate_mode} eq 'on') {
    if ($message =~ /[\d\w\s\r\n]+/) {
      $self->sendMessage(Msn::Bot::Translate::E2J($message));
    }
    else {
      $self->sendMessage(Msn::Bot::Translate::J2E($message));
    }
    return;
  }

  # TODO 
  # use Command pattern.
  # and generate command list in help message.
  elsif ($message =~ s/E2J\((.+?)\)/Msn::Bot::Translate::E2J($1)/eigs) {
    $self->sendMessage(uri_unescape($nickname)." said: $message");
  }
  elsif ($message =~ s/J2E\((.+?)\)/Msn::Bot::Translate::J2E($1)/eigs) {
    $self->sendMessage(uri_unescape($nickname)." said: $message");
  }
  elsif ($message =~ s/SEARCH\((.+?)\)/Msn::Bot::Google::search($1)/eigs) {
    $self->sendMessage($message);
  }
  elsif ($message =~ m/SPELL\((.+?)?\)/ig) {
    my $word = Msn::Bot::Google::spelling($1);
    $self->sendMessage($word || "spell '$1' is correct.");
  }
  elsif ($message =~ m/DICT\((.+?)\)/ig) {
    my $word = Msn::Bot::Dict::define($1);
    $self->sendMessage($word || "No Entry: $1");
  }
  elsif ($message =~ m/WORD\((.+?)\)/ig) {
    my $word = Msn::Bot::Dict::jedi($1, "yes", "ASCII");
    $word =~ s/\(\w\)//g;
    $self->sendMessage($word || "No Entry: $1");
  }
  elsif ($message =~ s/DATE\(\)/scalar localtime(time())/eig) {
    $self->sendMessage($message);
  }
  elsif ($message =~ s/HELP\(\)/E2J J2E SEARCH SPELL DICT WORD DATE/i) {
    $self->sendMessage($message);
  }
}

sub join_handler {
    my ($self, $username) = @_;
    
    print "join: @_\n";
}

sub answer_handler {
    my ($self, $username) = @_;
    
    print "answer: @_\n";
}


__END__

